##############################################################################
#
# PhotoIptcExif ProductExtension
#
# IPTC and EXIF reading support.
# A product extension to Ron Bickers' Photo Product 1.2.3, for Zope
#
# Copyright (c) Augusto Cabrera   All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Augusto Cabrera    ausum_studio@hotmail.com
#    February 23rd, 2002
#
##############################################################################
#
#
# MetadataHandler.py
#
# A helper module to PhotoIptcExif. In conjuntion with Photo Product and exif
# module, it extracts IPTC data from JPEGs and TIFFs during upload, and stores
# it as properties of the created object.
# 
# Thanks to Fredrik Lundh for sharing the core IPTC reader, and Robert F. 
# Tobler for the exif reader module.
#
# Ron Bickers' Photo Product:
# http://www.zope.org/Members/rbickers/Photo
#
##############################################################################



from PIL import Image, TiffImagePlugin, JpegImagePlugin, IptcImagePlugin, TiffTags
import exif
import StringIO
import string 



def iptcexifHandler(self, im, cp, justexif=0):
    """ Helper method. Given a read image, extract its IPTC and EXIF data, and update the 
    object's properties."""
    
    if cp:           
        createPropertiesSet(self)     # creates a set of IPTC properties regardless the 
                                      # file contains it or not.
    
    if justexif:            
        if str(im)[5:9] == 'Jpeg':
            try:
                exdata = exif.nparse(im.app['APP1'])
                if len(exdata) is not 0 :
                    setattr(self,'ED', self.exifHandler(2,ed=exdata))
                elif hasattr(self, 'ED'):
                    setattr(self,'ED', [])
            except:
                setattr(self,'ED', [])
                
        elif str(im)[5:9] == 'Tiff':
            try:        
                exdata = exif.mparse(im.fp)
                if len(exdata) is not 0 :
                    setattr(self,'ED', self.exifHandler(2,ed=exdata))
                elif hasattr(self, 'ED'):
                    setattr(self,'ED', [])
            except:
                setattr(self,'ED', [])
        else:
            setattr(self,'ED', [])            
    else:                
        try:
            if str(im)[5:9] in ('Jpeg', 'Tiff') and getiptcinfo(im) is not None:            
                iptcinfo = _getfilteredIPTCfields(im)
                for k, v in iptcinfo:
                    if k == 'Date_Created':                       
                        if len(v) == 8: fdate = v[:4]+'/'+v[4:6]+'/'+v[6:8] # back to Photoshop's style
                        else: fdate = v                             
                        setattr(self,'Date_Created', fdate)    # instead of the line below. Is it needed?               
                        # self.manage_changeProperties(Date_Created=fdate)                        
                    else:                        
                        setattr(self,k,v)   # Puts the content of fields in the object's properties
                        # self.manage_changeProperties({k:v})
                id = self.id
                # print "%s:  .......IPTC metadata found" % (id)
            else:
                iptcinfo = _getfilteredIPTCfields(im)
                for k, v in iptcinfo:
                    setattr(self,k,v)  
        except: pass
            
        
        if str(im)[5:9] == 'Jpeg':
            try:
                exdata = exif.nparse(im.app['APP1'])
                if len(exdata) is not 0 :
                    setattr(self,'ED', self.exifHandler(2,ed=exdata))
                elif hasattr(self, 'ED'):
                    setattr(self,'ED', [])
            except:
                setattr(self,'ED', [])
                
        elif str(im)[5:9] == 'Tiff':
            try:        
                exdata = exif.mparse(im.fp)
                if len(exdata) is not 0 :
                    setattr(self,'ED', self.exifHandler(2,ed=exdata))
                elif hasattr(self, 'ED'):
                    setattr(self,'ED', [])
            except:
                setattr(self,'ED', [])                
        else:
            setattr(self,'ED', [])



def get_info(img):
    typ = str(img)[5:9]
    try:
        if typ in ('Jpeg', 'Tiff') and getiptcinfo(img) is not None:            
            img.iptcinfo = _getfilteredIPTCfields(img)
    except: pass
    (img.width, img.height) = (img.size[0], img.size[1])  # Getting image sizes when
                                                          # parsing tiffs.

    if typ == 'Jpeg':
        try:
            img.exdata = exif.nparse(img.app['APP1'])
        except: pass        
    elif typ == 'Tiff':
        try:        
            img.exdata = exif.mparse(img.fp)
        except: pass




li = [  ((2, 120), 'Caption'),
        ((2, 122), 'Caption_Writer'), 
        ((2, 105), 'Headline'), 
        ((2, 40), 'Special_Instructions'),
        ((2, 25), 'Keywords'),
        ((2, 15), 'Category'),
        ((2, 20), 'Supplemental_Categories'), 
        ((2, 10), 'Urgency'), 
        ((2, 80), 'Byline'), 
        ((2, 85), 'Byline_Title'),
        ((2, 110), 'Credit'),
        ((2, 115), 'Source'), 
        ((2, 5), 'Object_Name'),
        ((2, 55), 'Date_Created'),
        ((2, 90), 'City'), 
        ((2, 95), 'Province_State'),
        ((2, 101), 'Country_Name'),
        ((2, 103), 'Original_Transmission_Reference'),
        ((2, 116), 'Copyright_Notice')]



def _getfilteredIPTCfields(im):    
    """Returns the list of IPTC fields used by Photoshop, discarding other existing
    fields within the image object. """
    info = getiptcinfo(im)
    nli = []
    for k,v in li:
        try:
            rdic = str(info[k])
            del info[k]
        except:
            rdic = ''
        nli.append((v,rdic))    
    return nli

    

def createPropertiesSet(self):
    """ Adds properties according to IPTC info, as supported by Photoshop.
    'Image URL' field is missing."""

    for x in [v for k, v in li]:                
        if x == 'Caption':
            try: self.manage_addProperty(x, '', 'text')   # Only these three fields qualify 
            except: pass                                  # to need a textarea
        elif x == 'Headline':
            try: self.manage_addProperty(x, '', 'text')
            except: pass
        elif x == 'Special_Instructions':
            try: self.manage_addProperty(x, '', 'text')
            except: pass
        else:
            try: self.manage_addProperty(x, '', 'string')
            except: pass
    
        


# 
#
# read IPTC info fields from PhotoShop TIFF and JPEG files
#
# by Fredrik Lundh, November 2001
#

class FakeImage:
    pass

def getiptcinfo(im):
    """ extract IPTC data from a PhotoShop JPEG or TIFF file. """

    data = None

    if isinstance(im, JpegImagePlugin.JpegImageFile):
        # extract the IPTC/NAA resource
        try:
            app = im.app["APP13"]
            if app[:14] == "Photoshop 3.0\x00":
                app = app[14:]
                # parse the image resource block
                offset = 0
                while app[offset:offset+4] == "8BIM":
                    offset = offset + 4
                    # resource code
                    code = JpegImagePlugin.i16(app, offset)
                    offset = offset + 2
                    # resource name (usually empty)
                    name_len = ord(app[offset])
                    name = app[offset+1:offset+1+name_len]
                    offset = 1 + offset + name_len
                    if offset & 1:
                        offset = offset + 1
                    # resource data block
                    size = JpegImagePlugin.i32(app, offset)
                    offset = offset + 4
                    if code == 0x0404:
                        # 0x0404 contains IPTC/NAA data
                        data = app[offset:offset+size]
                        break
                    offset = offset + size
                    if offset & 1:
                        offset = offset + 1
        except (AttributeError, KeyError):
            pass

    elif isinstance(im, TiffImagePlugin.TiffImageFile):
        # get raw data from the IPTC/NAA tag (PhotoShop tags the data
        # as 4-byte integers, so we cannot use the get method...)
        try:
            type, data = im.tag.tagdata[TiffImagePlugin.IPTC_NAA_CHUNK]
        except (AttributeError, KeyError):
            pass

    if data is None:
        return None # no properties

    # create an IptcImagePlugin object without initializing it
    im = FakeImage()
    im.__class__ = IptcImagePlugin.IptcImageFile

    # parse the IPTC information chunk
    im.info = {}
    im.fp = StringIO.StringIO(data)

    try:
        im._open()
    except (IndexError, KeyError):
        pass # expected failure

    return im.info

